# ANOVA DOS FACTORES

# PROBLEMA 1

# Fuente: Francesc Carmona: Ejercicios de Analisis de varianza con R

#(1) Ingreso de datos

plantas<-read.table(header=TRUE,text='
riego especie crecimiento
bajo	  vulgaris  9
bajo	  sativa    7
bajo	  vulgaris  11
bajo	  sativa    6
bajo    vulgaris  6
bajo    sativa    5
medio	  vulgaris  14
medio	  sativa    14
medio	  vulgaris  17
medio   sativa    17
medio   vulgaris  19
medio   sativa    15
alto	  vulgaris  28
alto	  sativa    44
alto	  vulgaris  31
alto    sativa    38
alto    vulgaris  32
alto    sativa    37
')

plantas
str(plantas)
attach(plantas)

# Graficos:

# diagramas de cajn:
boxplot(crecimiento ~ riego)
boxplot(crecimiento ~ especie)

# graficos de diseno de experimentos:
plot.design(plantas, fun = "mean")
plot.design(plantas, fun = "median")

# grficos de interaccin:
interaction.plot(especie, riego, crecimiento)
interaction.plot(riego, especie, crecimiento)

# resumenes  parciales:
tapply(crecimiento, riego, summary) # por riego
tapply(crecimiento, especie, summary) # por especie

# valores de crecimiento 
split(crecimiento, especie) #por especie:

# valores de crecimiento por riego:
split(crecimiento, riego)

# graficos de puntos especiales:
stripchart(crecimiento ~ riego)
stripchart(crecimiento ~ riego, method = "stack")
stripchart(crecimiento ~ riego, method = "jitter")

stripchart(crecimiento ~ especie, method = "stack")

#---------------------------------------------------------

#(2) 
# analisis de varianza:

modelo1<- lm(crecimiento ~ especie + riego + especie*riego)
anova(modelo1)

# Lo mismo se obtiene con  modelo1<- lm(crecimiento ~ especie*riego)

# o bien

modelo2 <- aov(crecimiento ~ especie + riego + especie*riego)
anova(modelo2)
summary(modelo2) # lo mismo de anova(modelo2)

model.tables(modelo2,type="means") # promedios parciales
model.tables(modelo2,type="means") # coeficientes estimados condicion usual
modelo2$coeff # coeficientes estimados condicion de R

#---------------------------------------------------------------

#(3) parametros estimados

summary(modelo1)
# o bien
modelo2$coeff # coeficientes estimados

# Matriz de diseno en esta situacion:
model.matrix(modelo1)

# Obs.: R estima los parametros con la condicion adicional alfa[1]=0, beta[1]=0, etc.

# Estimacion de los efectos de los niveles del factor (desviaciones resp del promedio):
model.tables(anova1) 
 
#--------------------------------------------------------------

#(4) Grafico normal

cajust<-fitted(modelo1) # valores ajustados
res<-resid(modelo1) # residuos
stres<-rstandard(modelo1) # residuos estandarizados
ajustados<-data.frame(riego,especie,crecimiento,cajust,res,stres)
ajustados

# promedios:
mean(crecimiento)
mean(cajust)
mean(res)
mean(stres)

# grafico de prob normal:
qqnorm(res) 
qqline(res)

#---------------------------------------------------

#(5) Pruebas de normalidad

# Shapiro-Wilks:

shapiro.test(res)

# Para Anderson-Darling y para Lilliefors debemos instalar la libreria "nortest"

library(nortest)
# si no la encuentra, corre antes: install.packages("nortest") 

# Anderson-Darling:

ad.test(res)

# Lilliefors:

lillie.test(res)

#------------------------------------------------

#(6) Pruebas de homosedasticidad

# Bartlett:

bartlett.test(res,g=especie) # entre especies
bartlett.test(res,g=riego) # entre riegos

# Levene:

library(car) # requerida para el test de Levene

leveneTest(res,g=especie)
leveneTest(res,g=riego)

#-----------------------------------------------

#(7) Grafico de residuos versus ajustados:

plot(cajust,res,main="Residuos versus ajustados")

#------------------------------------------------

#(8) Comparaciones de pares 

# prueba de Tuckey:

intervals = TukeyHSD(modelo2)
intervals

# prueba de a pares:

pairwise.t.test(crecimiento,riego, p.adj = "bonferroni")
# la prueba pairwise.t.test(crecimiento,especie, p.adj = "bonferroni")
# no tiene sentido porque "especie" resulto no significativa.

# Prueba Post Hoc de Duncan:

install.packages("DescTools")
library(DescTools)
PostHocTest(modelo2, method = "duncan")

########################################################

# PROBLEMA 2 (dos factores aleatorios)

#(1) ingreso de datos:

bacteria<-read.table(header=TRUE,text='
medio tiempo resp
c1  12  21
c1  12  22
c1  12  23
c1  12  28 
c1  12  20
c1  12  26
c1  17  37
c1  17  39 
c1  17  38
c1  17  38
c1  17  35
c1  17  36
c2  12  25
c2  12  26
c2  12  24
c2  12  25 
c2  12  29
c2  12  27
c2  17  31
c2  17  34
c2  17  29
c2  17  33
c2  17  30
c2  17  35
')

attach(bacteria)
str(bacteria)

#------------------------------------------------------------------

#(2) analisis de varianza:

modelo3<-aov(resp~medio + tiempo + medio*tiempo)
av<-anova(modelo3)

#-----------------------------------------------------------------

#(3) estimaciones de parametros

# calculos intermedios:
av
MSB<-av[2,3]
MSAB<-av[3,3]
MSer<-av[4,3]
MSB
MSAB
MSer

# La prueba F respectiva dio que los parametros Alfa son cero, por lo que
# no tiene sentido estimarlos 
# Sin embargo, si se tuvieran que estimar, se haria asi:

ypp<-tapply(resp, medio,mean)
a<-tapply(resp, medio,length)
a
a<-a[1]
a
ypp[1]
ypp[2]

# parametros estimados:
Mu<-mean(resp)
Mu
Alfa<-ypp-Mu
Alfa[1]
Alfa[2]

# si son significativas las varianzas de B, de AB y del error.
# Se estiman asi:

Sigma2B<-(MSB-MSer)/(a*n)
Sigma2AB<-(MSAB-MSer)/n
Sigma2<-MSer

Sigma2B
Sigma2AB
Sigma2

#############################################################
####################################################################33
