# ANOVA A UN FACTOR (B)

# Problema 1

#(1)
# Captura de particulas segun flujo de aire

captura<-c(755,730,708,721,745,756,785,775,786,811,800,869,806,
843,832,957,904,928,901,923)
captura

# Se debe generar factores y rplicas:
flujo<-gl(4, 5, labels = c("Bajo", "MedioBajo","MedioAlto","Alto"))
n<-length(flujo)
flujo # son niveles de un factor

nflujo=as.numeric(flujo) # numeros para algunos graficos
nflujo

# supongamos que el orden en que hicieron las corridas est dado por los siguientes 
# numeros, si se leen las capturaes de izq. a der., por filas.:

orden<-c(10,4,14,8,16,20,17,12,15,13,3,7,9,11,18,6,5,19,1,2)
# Con la instruccion split podemos separar los datos por flujo:

split(captura,flujo)

# resumen de los datos y el grafico:
summary(captura)
tapply(captura, flujo,summary) # Entrega summary separado por flujo

# cajagrama:
boxplot(captura ~ flujo,main="Numero de particulas",ylim=c(0,max(captura))) 

# grafico de puntos:
plot(nflujo,captura,type="p",xlab="Flujo",ylab="Particulas",main="Numero de particulas")

#--------------------------------------

#(2) analisis de varianza

# Asumiendo que la variable longitud sigue una distribucion normal con varianza comun para las tres
# poblaciones, la tabla del analisis de la varianza esta dado por

anova1 <- aov(captura ~ flujo)
summary(anova1) # obtener tabla de anova
# o bien
anova(anova1)

# Se pueden extraer valores individuales (usando identicos nombres que en la tabla):
anova(anova1)["Residuals", "Mean Sq"]
# o bien
anova(anova1)[2,2]

anova(anova1)["flujo","Mean Sq"]
anova(anova1)[,"Mean Sq"]
anova(anova1)["flujo",]

# otras opciones:
deviance(anova1) # suma de cuadrados de error
anova1$df.residual # g. lib de error

# estimacion insesgada de la varianza del modelo es
ECM<-deviance(anova1)/anova1$df.residual
ECM

#---------------------------------------

#(3) promedios

# promedio global. Estimacion de mu:
mean(captura)

# estimacion de los efectos de los niveles del factor (desviaciones resp del promedio):
model.tables(anova1) 
 
# promedio de las respuestas de los niveles:
model.tables(anova1, type = "mean")
# o bien 
mediasParc<-tapply(captura, flujo,mean)

# coeficientes ( con la condicion Suma de Alfa=0):
Mu<-mean(captura)
Mu

Alfa<-mediasParc-Mu
Alfa

# coeficientes entregados por R (con la condicion Alfa[1]=0):
anova1$coeff


#---------------------------------------------

#(4) Diseno

# Matriz de diseno en esta situacion:
model.matrix(anova1)

anova1$coeff # coeficientes estimados

#--------------------------------------------------------

#(5) Grafico normal

vajust<-fitted(anova1) # valores ajustados
res<-resid(anova1) # residuos
stres<-rstandard(anova1) # residuos estandarizados
ajustados<-data.frame(flujo,captura,vajust,res,stres,orden)
ajustados

# promedios:
mean(captura)
mean(vajust)
mean(res)
mean(stres)

(19/16)*var(res) # comparar con
ECM

# grafico de prob normal:
qqnorm(res) 
qqline(res)

# otra alternativa, que sirve para cualquier distribucion:
qqplot(rnorm(n), stres, main="Grafico Q-Q normal ",xlab="teoricos", ylab="observados")
abline(0,1)

#---------------------------------------------------

#(6) Pruebas de normalidad

# Shapiro-Wilks:

shapiro.test(res)

# Para Anderson-Darling y para Lilliefors debemos instalar la libreria "nortest"

library(nortest)
# si no la encuentra, corre antes: install.packages("nortest") 

# Anderson-Darling:

ad.test(res)

# Lilliefors:

lillie.test(res)

#------------------------------------------------

#(7) Pruebas de homosedasticidad

# Bartlett:

bartlett.test(res,g=flujo)

# Levene:

library(car) # requerida para el test de Levene

leveneTest(res,g=flujo)

#----------------------------------------------

#(8) Graficos

# residuos versus ajustados:

plot(vajust,res,main="Residuos versus ajustados")

# residuos versus el orden

plot(orden,res,main="Residuos segun orden")

#---------------------------------------------

#(9) Autocorrelaciones de primer orden

# Durbin Watson

# library(car) # requerida para el test de Durbin-Watson

# primero ordenar datos segun orden de experimentacion:
ajust<-ajustados[order(ajustados$orden),] # ordena data frame segn orden

# luego aplicar anova:
anova3<-aov(ajust$captura ~ ajust$flujo)

# por ultimo aplicar el test:
db<-durbinWatsonTest(anova3)
db

#-------------------------------------------

#(10) Contrastes ortogonales - caso 1

# groupoa a comparar (contrastes ortogonales):
c1 <- c(1, 1, -1, -1) # flujo menos de 190 vs. mas de 190
c2 <- c(1, -1, 0, 0) # 160 vs. 180
c3 <- c(0, 0,  1, -1) # 200 vs. 220

# combinar los tres contrastes en una matriz
C <- cbind(c1,c2,c3)

# definir que C corresponde a contrastes
contrasts(flujo) <- C

# analisis de varianza
anova4 <- aov(captura ~ flujo)
summary(anova4) # ver anova

# lo siguiente entrega la descomposicin de los 3 gl de flujo 
# en tres componentes con 1 gl c/u:

summary.aov(anova4, split=list(flujo=list("b y mb vs a y ma
 "=1, "160 vs. 180 " = 2, "200 vs. 220"=3))) 

#---------

#(11) Contrastes ortogonales - caso 2

# groupoa a comparar (contrastes ortogonales):
c1 <- c(-3,-1,1,3) # flujo menos de 190 vs. mas de 190
c2 <- c(2,-2,-2,2) # 160 vs. 180
c3 <- c(1,-3,3,-1) # 200 vs. 220

# combinar los tres contrastes en una matriz
C <- cbind(c1,c2,c3)

# definir que C corresponde a contrastes
contrasts(flujo) <- C

# analisis de varianza
anova4 <- aov(captura ~ flujo)
summary(anova4) # ver anova

# lo siguiente entrega la descomposicin de los 3 gl de flujo 
# en tres componentes con 1 gl c/u:

summary.aov(anova4, split=list(flujo=list("lineal"=1, "cuadratico" = 2, "cubico"=3))) 

# -------------------------------------------------------------

$(12)  Pruebas de Tukey

tukey<-TukeyHSD(anova1)
tukey

#################################################################

# Problema 2

#(13)
# Resistencia

resistencia<-c(98,97,99,96,
91,90,93,92,
96,95,97,95,
95,96,99,98)
resistencia
n<-length(resistencia)

# Se debe generar factores y rplicas:
telar<-gl(4, 4, labels = c("telar 1", "telar 2","telar 3","telar 4"))
telar # son niveles de un factor
ntelar=as.numeric(telar) # numeros
ntelar

split(resistencia,telar)

# resumen de los datos y el grafico:
summary(resistencia)
tapply(resistencia, telar,summary) # Entrega summary separado por telar

# cajagrama:
boxplot(resistencia ~ telar,main="Resistencia de telas") 

# grafico de puntos:
plot(ntelar,resistencia,type="p",xlab="Telar",ylab="Resistencia",main="Resistencia de telas")

#---------------------

# analisis de varianza

# Asumiendo que la variable longitud sigue una distribucion normal con varianza comun para las tres
# poblaciones, la tabla del analisis de la varianza esta dado por

anova1 <- aov(resistencia ~ telar)
summary(anova1) # obtener tabla de anova

# Se pueden extraer valores individuales (usando identicos nombres que en la tabla):

# estimacion de la varianza del error (MSerr) :
sigma2<-anova(anova1)["Residuals","Mean Sq"]
sigma2

# estimacion de la varianza de tratamientos SigmaTau:
MStrat<-anova(anova1)["telar","Mean Sq"]
MStrat
sigmaTau2<-(MStrat-sigma2)/n
sigmaTau2

############################################################

# Problema 3 (continuacion del problema 1)

#(14) Prueba de Kruskal-Wallis

# correr los datos Problema 1 :

# Captura de particulas segun flujo de aire

captura<-c(755,730,708,721,745,756,785,775,786,811,800,869,806,
843,832,957,904,928,901,923)
captura
n<-length(flujo)

# Se debe generar factores y rplicas:
flujo<-gl(4, 5, labels = c("B", "MB","MA","A"))
flujo # son niveles de un factor

# prueba de Kruskal-Wallis no parametrica para efecto de tratamientos:
kruskal.test(captura ~ flujo) 

#-----------------------------------------------------

#############################################################

