# Regresion lineal multiple

# Problema 1

#(1) ingreso de datos:

TCDD<-read.table(header=TRUE,text='
   dioxina   loga adesde edad masa
1     20.2 270.53      7   37 23.0
2     23.0 278.25      6   39 28.2
3     20.4 277.03      7   37 21.4
4     20.8 243.72      1   50 32.4
5     20.6 280.28      4   36 21.8
6     18.7 282.31      3   43 23.0
7     19.5 269.31      2   44 26.6
8     22.2 320.91      9   33 20.9
9     21.4 245.34      6   39 25.3
10    23.3 270.53      7   37 27.7
11    27.7 278.25      6   39 32.5
12    22.9 245.34      3   46 31.4
13    23.1 243.72      1   50 33.4
14    20.0 282.31      3   43 25.9
15    24.2 269.31      2   44 32.4
16    26.0 320.91      9   33 25.9
17    26.9 245.34      6   39 32.0
18    27.4 270.53      7   37 32.4
19    30.7 278.25      6   39 34.5
20    24.4 277.03      7   37 25.7
21    26.3 245.34      3   46 36.6
22    24.4 243.72      1   50 36.9
23    22.5 280.28      4   36 25.8
24    23.7 282.31      3   43 31.9
25    29.6 269.31      2   44 38.0
26    30.6 320.91      9   33 30.9
27    29.5 245.34      6   39 35.0
28    30.2 270.53      7   37 35.6
29    29.0 277.03      7   37 30.6
30    27.6 280.28      4   36 31.1
31    28.0 282.31      3   43 36.0
32    34.7 320.91      9   33 35.4
')   

attach(TCDD)
head(TCDD) # ver primeras 6 filas
tail(TCDD) # ver ultimas 6
TCDD # ver todo
View(TCDD) # ver como planilal de calculo
TCDD2<-edit(TCDD) # permite efectuar cambios
head(TCDD2)
rm(TCDD2) # elimina de la memoria
n<-nrow(TCDD)

# analisis exploratorio bivariado:

pairs(dioxina~loga+adesde+edad+masa,main="Matriz de diagramas de dispersion")

# analisis exploratorio univariado por variable:

summary(TCDD) # resumen

# ------------------------------------------------------

#(2) Modelo de regresion y tests t por cada regresor:

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) # linear model
summary(lmDioxina)

# ------------------------------------------------------

#(3) Analisis de varianza:

anova(lmDioxina) # antes se debe correr lmDioxina<-lm(...

o bien, equivalentemente
summary.aov(lmDioxina)

# Se puede extraer informacin de esta tabla. Por ejemplo:
anova(lmDioxina)["Residuals", "Sum Sq"] # se debe usar los nombres tal como estan en la tabla Anova
anova(lmDioxina)["loga","Mean Sq"]
anova(lmDioxina)[,"Sum Sq"]
anova(lmDioxina)["loga",]

# Prueba global de regresion (todos los regresores en conjunto):

SSreg<-sum(anova(lmDioxina)[1:4,"Sum Sq"])
SSreg
glreg<-4
MSreg<-SSreg/4
MSreg
MSerr<-anova(lmDioxina)["Residuals","Mean Sq"]
MSerr
glerr<-anova(lmDioxina)["Residuals","Df"]
Freg<-MSreg/MSerr
Freg
pvalorReg<-1-pf(Freg,glreg,glerr)
pvalorReg

# ----------------------------------------------------

#(4) coeficiente de determinacion 

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) # linear model
summary(lmDioxina)

# --------------------------------------------------

#(5) pruebas t individuales

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) # linear model
summary(lmDioxina)

#####################################################

# Pruebas de cumplimiento de supuestos:

#(6) normalidad

# Grafico de prob normal

vajust<-fitted(lmDioxina) # valores ajustados
res<-resid(lmDioxina) # residuos
stres<-rstandard(lmDioxina) # residuos estandarizados

qqnorm(res) 
qqline(res)

# otra alternativa, que sirve para cualquier distribucion:
qqplot(rnorm(n), stres, main="Grafico Q-Q normal ",xlab="teoricos", ylab="observados")
abline(0,1)

# Pruebas de normalidad

# Shapiro-Wilks:

shapiro.test(res)

# Para Anderson-Darling y para Lilliefors debemos instalar la libreria "nortest"

library(nortest)
# si no la encuentra, correr antes: install.packages("nortest") 

# Anderson-Darling:

ad.test(res)

# Lilliefors:

lillie.test(res)

#------------------------------------------------

#(7) Pruebas de homosedasticidad

# prueba de Breusch-Pagan

library(lmtest) # requerida para el test de B-P

bptest(lmDioxina)

#-------------------------------------------------

#(8) No Autocorrelaciones de primer orden

# Durbin Watson

library(car) # requerida para el test de Durbin-Watson

db<-durbinWatsonTest(lmDioxina)
db

#-----------------------------------------------

#(9) No multicolinealidad

# varianca Inflation Factors - el cuociente entre la varianza del modelo completo
# y la varianza del modelo con solo la variable estudiada.
# Un valor de 1 indica absoluta falta de multicolinealidad. 
# A partir del valor 4 seria indicador de la presencia de multicolinelidad

# library(car) # requerida para vif

vif(lmDioxina)

# -----------------------------------------------

# Graficos

# residuos versus ajustados:

plot(vajust,res,main="Residuos versus ajustados")
abline(a=0,b=0)

# residuos versus el orden

plot(dioxina,res,main="Residuos segun orden")
abline(0,0)

#---------------------------------------------

#(10) sumas de cuadrados parciales

# Observar que son distintos:

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde+loga+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde+masa+loga) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde+loga+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde+edad+masa) 
anova(lmDioxina)

lmDioxina<-lm(dioxina~adesde)
anova(lmDioxina)

# La explicacin es que las SS de los regresores (desde arriba abajo) 
# son parciales dados los anteriores (de arriba).

###########################################################

#(11) Modelos alternativos

# recordar:

lmDioxina<-lm(dioxina~loga+adesde+edad+masa) 
summary(lmDioxina)

# edad y masa aparecen como significativos.
# Probaremos tres modelos: mod 1 - edad y masa
# mod 2 - edad y masa mas loga
# mod 3 - edad y masa mas adesde

# -------------------------------------

# mod 1

mod1<-lm(dioxina~edad+masa) 
summary(mod1) # ambas significativas

mod2<-lm(dioxina~loga+edad+masa)
summary(mod2) # solo edad+masa signif

mod3<-lm(dioxina~adesde+edad+masa) 
summary(mod3) # solo edad+masa signif

# Nos quedamos con el modelo 1

# ---------------------------------------------

# Verificacion de supuestos:

# Grafico de prob normal

res1<-resid(mod1) # residuos

qqnorm(res1) 
qqline(res1)

# ---------------------------------------------

# Prueba de normalidad de Shapiro-Wilks:

shapiro.test(res1)

# ---------------------------------------------

# Prueba de homosedasticidad de Breusch-Pagan

# library(lmtest) # requerida para el test de B-P

bptest(mod1)

# ---------------------------------------------

# No multicolinealidad

# library(car) # requerida para vif

vif(mod1)

# ---------------------------------------------
##################################################

#(12) Prediccion

# Intervalos de confianza para la dioxinauccion esperada

predict(mod1,interval="confidence")

# ------------------------------------------------

# Intervalos de prediccion para la dioxinauccion

predict(mod1,interval="prediction")

# ------------------------------------------------

# Queremos predecir valors de dioxinauccion para los valores de 
# (edad,masa)=(40,33) ; (42,36) ; (44,42):

nuevaTemp<-data.frame(edad=c(40,42,44),masa=c(33,36,42))
nuevaTemp

predict(mod1,nuevaTemp,interval="confidence")

predict(mod1,nuevaTemp,interval="prediction")

######################################################


