#  CALIBRACION

#
# Arsenic example (simple linear regression with replication)
#

# Inverting a prediction interval for an individual response
arsenic.lm <- lm(measured ~ actual, data = arsenic)
plotFit(arsenic.lm, interval = "prediction", shade = TRUE, 
        col.pred = "lightblue")
(cal <- calibrate(arsenic.lm, y0 = 3, interval = "inversion"))
abline(h = 3)
segments(cal$estimate, 3, cal$estimate, par()$usr[3])
arrows(cal$lower, 3, cal$lower, par()$usr[3])
arrows(cal$upper, 3, cal$upper, par()$usr[3])

#
# Crystal weight example (simple linear regression)
#

# Inverting a confidence interval for the mean response
crystal.lm <- lm(weight ~ time, data = crystal)
plotFit(crystal.lm, interval = "confidence", shade = TRUE,
        col.conf = "lightblue")
(cal <- calibrate(crystal.lm, y0 = 8, interval = "inversion", 
                  mean.response = TRUE))
abline(h = 8)
segments(cal$estimate, 8, cal$estimate, par()$usr[3])
arrows(cal$lower, 8, cal$lower, par()$usr[3])
arrows(cal$upper, 8, cal$upper, par()$usr[3])

# Wald interval and approximate standard error based on the delta method
calibrate(crystal.lm, y0 = 8, interval = "Wald", mean.response = TRUE)


###########################################################



# R function to do classical calibration
# input the x and y vectors
# returns a data.frame defined in the main function with
# vectors xpre and conf which are the predicted x values and
# the single confidence interval about x
calib <- function(indep,dep)
{

	# generate the model y on x and get out predicted values for x
	reg <- lm(dep~indep)
	xpre <- (dep-coef(reg)[1])/coef(reg)[2]

	# generate a confidence
	yyat <- ((dep-predict(reg))^2)
	sigyyat <- ((sum(yyat)/(length(dep)-2))^0.5)
	term1 <- sigyyat/coef(reg)[2]
	sigxxbar <- sum((indep-mean(indep))^2)
	denom <- sigxxbar * ((coef(reg)[2])^2)
	conf <- 
abs(((1+(1/length(dep))+(((dep-mean(dep))^2)/denom))^2)*term1)

return(xpre, conf)
}

##############################################################


# R function to do classical calibration
# input the x and y vectors and the value of y to make an
# estimate for
# val is a value of dep
# returns the calibrated value of x and it's
# single confidence interval about x
# form of calibration particularly suited to jackknifing
calib <- function(indep, dep, val)
{

	# generate the model y on x and get out predicted values for x
	reg <- lm(dep~indep)
	xpre <- (val - coef(reg)[1])/coef(reg)[2]

	# generate a confidence
	yyat <- ((dep - predict(reg))^2)
	sigyyat <- ((sum(yyat)/(length(dep)-2))^0.5)
	term1 <- sigyyat/coef(reg)[2]
	sigxxbar <- sum((indep - mean(indep))^2)
	denom <- sigxxbar * ((coef(reg)[2])^2)
	conf <- abs(((1+(1/length(dep))+(((val -
mean(dep))^2)/denom))^2)*term1)

return(xpre, conf)
}


# R function to do classical calibration
# input the x and y vectors
# returns a data.frame defined in the main function with
# vectors xpre and conf which are the predicted x values and
# the single confidence interval about x
calib <- function(indep,dep)
{

	# generate the model y on x and get out predicted values for x
	reg <- lm(dep~indep)
	xpre <- (dep-coef(reg)[1])/coef(reg)[2]

	# generate a confidence
	yyat <- ((dep-predict(reg))^2)
	sigyyat <- ((sum(yyat)/(length(dep)-2))^0.5)
	term1 <- sigyyat/coef(reg)[2]
	sigxxbar <- sum((indep-mean(indep))^2)
	denom <- sigxxbar * ((coef(reg)[2])^2)
	conf <- 
abs(((1+(1/length(dep))+(((dep-mean(dep))^2)/denom))^2)*term1)

return(xpre, conf)
}




v
# R function to do classical calibration
# input the x and y vectors and the value of y to make an
# estimate for
# val is a value of dep
# returns the calibrated value of x and it's
# single confidence interval about x
# form of calibration particularly suited to jackknifing
calib <- function(indep, dep, val)
{

	# generate the model y on x and get out predicted values for x
	reg <- lm(dep~indep)
	xpre <- (val - coef(reg)[1])/coef(reg)[2]

	# generate a confidence
	yyat <- ((dep - predict(reg))^2)
	sigyyat <- ((sum(yyat)/(length(dep)-2))^0.5)
	term1 <- sigyyat/coef(reg)[2]
	sigxxbar <- sum((indep - mean(indep))^2)
	denom <- sigxxbar * ((coef(reg)[2])^2)
	conf <- abs(((1+(1/length(dep))+(((val -
mean(dep))^2)/denom))^2)*term1)

return(xpre, conf)
}


# R function to do classical calibration
# input the x and y vectors
# returns a data.frame defined in the main function with
# vectors xpre and conf which are the predicted x values and
# the single confidence interval about x
calib <- function(indep,dep)
{

	# generate the model y on x and get out predicted values for x
	reg <- lm(dep~indep)
	xpre <- (dep-coef(reg)[1])/coef(reg)[2]

	# generate a confidence
	yyat <- ((dep-predict(reg))^2)
	sigyyat <- ((sum(yyat)/(length(dep)-2))^0.5)
	term1 <- sigyyat/coef(reg)[2]
	sigxxbar <- sum((indep-mean(indep))^2)
	denom <- sigxxbar * ((coef(reg)[2])^2)
	conf <- 
abs(((1+(1/length(dep))+(((dep-mean(dep))^2)/denom))^2)*term1)

return(xpre, conf)
}


###################################################################

install.packages("investr")
library(investr)
mod <- lm(measured ~ actual, data = arsenic)
(res <- calibrate(mod, y0 = 3, interval = "inversion", level = 0.9))
## Figure 1
plotFit(mod, interval = "prediction", level = 0.9, shade = TRUE, col.pred = "skyblue")
abline(h = 3, v = c(res$lower, res$estimate, res$upper), lty = 2)


calibrate(mod, y0 = 3, interval = "inversion", level = 0.9)
calibrate(badfit, y0 = 10, level = 0.9)
c(3.17, 3.09, 3.16)


