# MEDIA MOVIL Y SUAVIZAMIENTO EXPONENCIAL (manual)

#(1) Problema 1 URGENCIAS HOSPITALARIAS

install.packages("readxl")
library(readxl)

Urgencias<-read_excel(file.choose(),col_names=TRUE) # leer archivo excel
head(Urgencias)
str(Urgencias)
attach(Urgencias)
n<-length(urgencias)
obs<-1:n # crear nueva variable

plot(obs,urgencias,type="l",xlab="mes", ylab="urgencias",main="Urgencias por mes",ylim=c(0,max(urgencias)))

########################################################################

#(5) MEDIA MOVIL

y<-urgencias # para todos los MM

# a) MM(K1):

K1<-3

z3<-numeric(n)

for (j in 2:(n-1)) {
	z3[j]<-(y[j-1]+y[j]+y[j+1])/K1
}

z3[1]<-NA
z3[n]<-NA

plot(obs,y,type="l",col="grey")
lines(obs,z3,type="l",col="red")

#---------------------

# b) MM(K5)

K5<-5

z5<-numeric(n)

for (j in 3:(n-2)) {
	z5[j]<-(y[j-2]+y[j-1]+y[j]+y[j+1]+y[j+2])/K5
}

z5[1]<-NA 
z5[2]<-NA
z5[n]<-NA
z5[n-1]<-NA

plot(obs,y,type="l",col="grey")
lines(obs,z5,type="l",col="red")

# ----------------------------

# b) MM(K6)

K6<-6

z6<-numeric(n)

for (j in 3:(n-2)) {
	z6[j]<-(y[j-2]+y[j-1]+y[j]+y[j+1]+y[j+2]+y[j+3])/K6
}

z6[1]<-NA
z6[2]<-NA
z6[n]<-NA
z6[n-1]<-NA
z6[n-2]<-NA

plot(obs,y,type="l",col="grey")
lines(obs,z6,type="l",col="blue")

# ----------------------------

# c) MM(K3)

K3<-12

z12<-numeric(n)

for (j in 7:(n-5)) {
	z12[j]<-(y[j-6]+y[j-5]+y[j-4]+y[j-3]+y[j-2]+y[j-1]+y[j]+y[j+1]+y[j+2]+y[j+3]+y[j+4]+y[j+5])/K3
}

# otra forma de manejar los extremos:
z12[1]<-NA
z12[2]<-NA
z12[3]<-NA
z12[4]<-NA
z12[5]<-NA
z12[6]<-NA

z12[n]<-NA
z12[n-1]<-NA
z12[n-2]<-NA
z12[n-3]<-NA
z12[n-4]<-NA

plot(obs,y,type="l",col="grey")
lines(obs,z12,type="l",col="blue")

#------------------------------

plot(obs,y,type="l",col="grey",main="Suavizamiento de media mvil",lwd=2)
lines(obs,z3,type="l",col="red",lwd=2)
lines(obs,z5,type="l",col="blue",lwd=2)
lines(obs,z12,type="l",col="black",lwd=2)

legend("topleft",c("y","z3","z5","z12"),col=c("grey","red","blue","black"),lwd=2)

#######################################################################

# funcion media movil orden k: 

MM<-function(y,k){
  n<-length(y)
  z<-numeric(n)
  # desface:
  k2<-k%/%2
  res<-k%%2
  if (res==0){ # determinar si el orden es par o impar
     k1<-k2-1  # orden par
     }else {   # orden impar
      k1<-k2
      }
  # media movil:
  for (t in (k1+1):(n-k2)) {
     z[t]<-0  
     for (j in (t-k1):(t+k2)){
        z[t]<-z[t]+y[j]
     }
     z[t]<-z[t]/k 
     }
  # extremo inicial:
  lim<-k1+1 
  for (t in 1:k1) {
     z[t]<-NA     
     }
    
  # extremo final:
  lim<-n-k1-k2+1
  for (t in (n-k2+1):n) {
     z[t]<-NA    
     }

  # resultado:
     MM<-return(z)
} # fin de la funcion

w2<-MM(y,2)
w6<-MM(y,6)
w12<-MM(y,12)
mn<-"Suavizamiento de media mvil orden"
plot(obs,y,type="l",col="grey",main=mn)
lines(obs,w,type="l",col="blue")

# Graficos
plot(obs,y,type="l",col="grey",main=mn,lwd=2)
lines(obs,w2,type="l",col="red",lwd=2)
lines(obs,w6,type="l",col="blue",lwd=2)
lines(obs,w12,type="l",col="black",lwd=2)

legend("topleft",c("y","w2","w6","w12"),col=c("grey","red","blue","black"),lwd=2)
#---------------------------------------------------------------

# Media movil orden k usando el comando de R

install.packages("zoo")
library(zoo)

mm3<-rollmean(y, k = 3 , fill = NA, align = "center")
mm5<-rollmean(y, k = 5 , fill = NA, align = "center")
mm7<-rollmean(y, k = 7 , fill = NA, align = "center")
mm12<-rollmean(y, k = 12 , fill = NA, align = "center")


plot(obs,y,type="l",col="grey",main="Urgencias",lwd=2)
lines(obs,mm3,type="l",col="red",lwd=2)
lines(obs,mm5,type="l",col="blue",lwd=2)
lines(obs,mm7,type="l",col="green",lwd=2)
lines(obs,mm12,type="l",col="violet",lwd=2)

legend("bottomright",c("original","mm3","mm5","mm7","mm12"),text.col=c("grey","red","blue","green","violet"))

###############################################################3

#(6) SUAVIZAMIENTO EXPONENCIAL

# a) SE(a1):

y<-urgencias

a1<-0.8

z1<-numeric(n)
z1<-y[1]

for (j in 2:n) {
	z1[j]<-a1*y[j]+(1-a1)*z1[j-1]
}

plot(obs,y,type="l",col="grey")
lines(obs,z1,type="l",col="red")

#------------------------

# b) SE(a2):

a2<-0.5

z2<-numeric(n)
z2[1]<-y[1]

for (j in 2:n) {
	z2[j]<-a2*y[j]+(1-a2)*z2[j-1]
}

plot(obs,y,type="l",col="grey")
lines(obs,z2,type="l",col="blue")

#-------------------------

# b) SE(a3):

a3<-0.2

z4<-numeric(n)
z4[1]<-y[1]

for (j in 2:n) {
	z4[j]<-a3*y[j]+(1-a3)*z4[j-1]
}

plot(obs,y,type="l",col="grey")
lines(obs,z4,type="l",col="black")

#------------------------

plot(obs,y,type="l",col="grey",main="Suavizamiento exponencial")
lines(obs,z1,type="l",col="red")
lines(obs,z2,type="l",col="blue")
lines(obs,z4,type="l",col="black")

legend("bottomright",c("y","z1","z2","z4"),col=c("grey","red","blue","black"),lwd=1)

#-----------------------------------------------------

# funcion suavizamiento exponencial

y<-urgencias 

SE<-function(y,a){
     z<-numeric(n+1)
     z<-y[1]
     for (j in 2:n) {   # suavizamiento exponencial
	  z[j]<-a*y[j]+(1-a)*z[j-1]
        }
      z[n+1]<-a*y[n]+(1-a)*z[n] # prediccion
      return(c(z))
}



u<-SE(y,0.75)
u

obs1<-c(obs,n+1)
plot(obs,y,type="l",col="grey",xlim<-c(0,n+1))
lines(obs1,u,type="l",col="blue")
length(obs1)
length(u)








####################################################################
